@@ -390,7 +390,7 @@ class ApplicationEngine(ExecutionEngine):
         return 1
 
     @staticmethod
-    def Run(script, container=None, exit_on_error=False):
+    def Run(script, container=None, exit_on_error=False, gas=Fixed8.Zero(), test_mode=True):
         """
         Runs a script in a test invoke environment
 
@@ -423,8 +423,8 @@ class ApplicationEngine(ExecutionEngine):
             container=container,
             table=script_table,
             service=service,
-            gas=Fixed8.Zero(),
-            testMode=True,
+            gas=gas,
+            testMode=test_mode,
             exit_on_error=exit_on_error
         )
 
@@ -434,8 +434,10 @@ class ApplicationEngine(ExecutionEngine):
 
         try:
             success = engine.Execute()
+            engine.testMode = True
             service.ExecutionCompleted(engine, success)
         except Exception as e:
+            engine.testMode = True
             service.ExecutionCompleted(engine, False, e)
 
         for event in service.events_to_dispatch:
