@@ -11,6 +11,7 @@ from neo.VM.InteropService import Array, Struct, StackItem, CollectionMixin, Map
 from neocore.UInt160 import UInt160
 from neo.Settings import settings
 from neo.VM.VMFault import VMFault
+from neo.Prompt.vm_debugger import VMDebugger
 from logging import DEBUG as LOGGING_LEVEL_DEBUG
 
 
@@ -21,7 +22,7 @@ class ExecutionEngine():
     _ScriptContainer = None
     _Crypto = None
 
-    _VMState = VMState.BREAK
+    _VMState = None
 
     _InvocationStack = None
     _EvaluationStack = None
@@ -38,6 +39,9 @@ class ExecutionEngine():
     log_file = None
     _is_write_log = False
 
+    _debug_map = None
+    _vm_debugger = None
+
     def write_log(self, message):
         """
         Write a line to the VM instruction log file.
@@ -95,6 +99,7 @@ class ExecutionEngine():
         return self._ExecutedScriptHashes
 
     def __init__(self, container=None, crypto=None, table=None, service=None, exit_on_error=False):
+        self._VMState = VMState.BREAK
         self._ScriptContainer = container
         self._Crypto = crypto
         self._Table = table
@@ -110,6 +115,13 @@ class ExecutionEngine():
     def AddBreakPoint(self, position):
         self.CurrentContext.Breakpoints.add(position)
 
+    def LoadDebugInfo(self, debug_map=None):
+        if debug_map:
+            self._debug_map = debug_map
+            for b in self._debug_map['breakpoints']:
+                self.AddBreakPoint(b)
+#                print("breakpoint %s " % b)
+
     def Dispose(self):
         while self._InvocationStack.Count > 0:
             self._InvocationStack.Pop().Dispose()
@@ -132,6 +144,7 @@ class ExecutionEngine():
         estack = self._EvaluationStack
         istack = self._InvocationStack
         astack = self._AltStack
+#        print("VM STATE %s " % self._VMState)
 
         if opcode > PUSH16 and opcode != RET and context.PushOnly:
             return self.VM_FAULT_and_report(VMFault.UNKNOWN1)
@@ -872,9 +885,10 @@ class ExecutionEngine():
                 return self.VM_FAULT_and_report(VMFault.UNKNOWN_OPCODE, opcode)
 
         if self._VMState & VMState.FAULT == 0 and self.InvocationStack.Count > 0:
-
-            if self.CurrentContext.InstructionPointer in self.CurrentContext.Breakpoints:
-                self._VMState |= VMState.BREAK
+            if len(self.CurrentContext.Breakpoints):
+                if self.CurrentContext.InstructionPointer in self.CurrentContext.Breakpoints:
+                    self._vm_debugger = VMDebugger(self)
+                    self._vm_debugger.start()
 
     def LoadScript(self, script, push_only=False):
 
@@ -891,7 +905,6 @@ class ExecutionEngine():
 
     def StepInto(self):
         if self._InvocationStack.Count == 0:
-            logger.info("INVOCATION COUNT IS 0, HALT")
             self._VMState |= VMState.HALT
 
         if self._VMState & VMState.HALT > 0 or self._VMState & VMState.FAULT > 0:
