@@ -8,6 +8,7 @@ import psutil
 import traceback
 import logging
 import sys
+from time import sleep
 from logzero import logger
 from prompt_toolkit import prompt
 from prompt_toolkit.contrib.completers import WordCompleter
@@ -111,6 +112,7 @@ class PromptInterface(object):
                 'state',
                 'config debug {on/off}',
                 'config sc-events {on/off}',
+                'config maxpeers {num_peers}',
                 'build {path/to/file.py} (test {params} {returntype} {needs_storage} {needs_dynamic_invoke} {test_params})',
                 'load_run {path/to/file.avm} (test {params} {returntype} {needs_storage} {needs_dynamic_invoke} {test_params})',
                 'import wif {wif}',
@@ -243,8 +245,7 @@ class PromptInterface(object):
                 try:
                     self.Wallet = UserWallet.Open(path, password_key)
 
-                    self._walletdb_loop = task.LoopingCall(self.Wallet.ProcessBlocks)
-                    self._walletdb_loop.start(1)
+                    self.start_wallet_loop()
                     print("Opened wallet at %s" % path)
                 except Exception as e:
                     print("Could not open wallet: %s" % e)
@@ -294,17 +295,23 @@ class PromptInterface(object):
                     return
 
                 if self.Wallet:
-                    self._walletdb_loop = task.LoopingCall(self.Wallet.ProcessBlocks)
-                    self._walletdb_loop.start(1)
+                    self.start_wallet_loop()
 
             else:
                 print("Please specify a path")
 
+    def start_wallet_loop(self):
+        self._walletdb_loop = task.LoopingCall(self.Wallet.ProcessBlocks)
+        self._walletdb_loop.start(1)
+
+    def stop_wallet_loop(self):
+        self._walletdb_loop.stop()
+        self._walletdb_loop = None
+
     def do_close_wallet(self):
         if self.Wallet:
             path = self.Wallet._path
-            self._walletdb_loop.stop()
-            self._walletdb_loop = None
+            self.stop_wallet_loop()
             self.Wallet.Close()
             self.Wallet = None
             print("Closed wallet %s" % path)
@@ -383,7 +390,9 @@ class PromptInterface(object):
             print("Import of '%s' not implemented" % item)
 
     def do_build(self, arguments):
+        Blockchain.Default().Pause()
         BuildAndRun(arguments, self.Wallet)
+        Blockchain.Default().Resume()
 
     def do_load_n_run(self, arguments):
         LoadAndRun(arguments, self.Wallet)
@@ -532,7 +541,11 @@ class PromptInterface(object):
         elif item == 'claim':
             ClaimGas(self.Wallet, True, arguments[1:])
         elif item == 'rebuild':
-            self.Wallet.Rebuild()
+            self.stop_wallet_loop()
+            try:
+                self.Wallet.Rebuild()
+            finally:
+                self.start_wallet_loop()
             try:
                 item2 = int(get_arg(arguments, 1))
                 if item2 and item2 > 0:
@@ -593,7 +606,7 @@ class PromptInterface(object):
 
     def show_nodes(self):
         if len(NodeLeader.Instance().Peers) > 0:
-            out = ""
+            out = "Total Connected: %s " % len(NodeLeader.Instance().Peers)
             for peer in NodeLeader.Instance().Peers:
                 out += "Peer %s - IO: %s\n" % (peer.Name(), peer.IOStats())
             print_tokens([(Token.Number, out)], self.token_style)
@@ -885,6 +898,20 @@ class PromptInterface(object):
             else:
                 print("Cannot configure VM instruction logging. Please specify on|off")
 
+        elif what == 'maxpeers':
+            try:
+                c1 = int(get_arg(args, 1).lower())
+                num_peers = int(c1)
+                if num_peers > 0:
+                    old_max_peers = settings.CONNECTED_PEER_MAX
+                    settings.set_max_peers(num_peers)
+                    NodeLeader.Instance().OnUpdatedMaxPeers(old_max_peers, num_peers)
+                    print("set max peers to %s " % num_peers)
+                else:
+                    print("Please specify integer greater than zero")
+            except Exception as e:
+                print("Cannot configure max peers. Please specify an integer greater than 0")
+
         else:
             print(
                 "Cannot configure %s try 'config sc-events on|off', 'config debug on|off', 'config sc-debug-notify on|off' or 'config vm-log on|off'" % what)
@@ -893,7 +920,7 @@ class PromptInterface(object):
         dbloop = task.LoopingCall(Blockchain.Default().PersistBlocks)
         dbloop.start(.1)
 
-        Blockchain.Default().PersistBlocks()
+#        Blockchain.Default().PersistBlocks()
 
         tokens = [(Token.Neo, 'NEO'), (Token.Default, ' cli. Type '),
                   (Token.Command, '\'help\' '), (Token.Default, 'to get started')]
@@ -1014,6 +1041,10 @@ def main():
     parser.add_argument("--datadir", action="store",
                         help="Absolute path to use for database directories")
 
+    # peers
+    parser.add_argument("--maxpeers", action="store", default=5,
+                        help="Max peers to use for P2P Joining")
+
     # Show the neo-python version
     parser.add_argument("--version", action="version",
                         version="neo-python v{version}".format(version=__version__))
@@ -1043,6 +1074,9 @@ def main():
     if args.datadir:
         settings.set_data_dir(args.datadir)
 
+    if args.maxpeers:
+        settings.set_max_peers(args.maxpeers)
+
     # Instantiate the blockchain and subscribe to notifications
     blockchain = LevelDBBlockchain(settings.chain_leveldb_path)
     Blockchain.RegisterBlockchain(blockchain)
@@ -1055,7 +1089,7 @@ def main():
     cli = PromptInterface()
 
     # Run things
-    reactor.suggestThreadPoolSize(15)
+#    reactor.suggestThreadPoolSize(15)
     reactor.callInThread(cli.run)
     NodeLeader.Instance().Start()
 
